Index: ChangeLog
===================================================================
RCS file: /cvs/kaffe/kaffe/ChangeLog,v
retrieving revision 1.1233
diff -w -u -p -r1.1233 ChangeLog
--- ChangeLog	11 Dec 2002 01:43:21 -0000	1.1233
+++ ChangeLog	29 Dec 2002 17:24:35 -0000
@@ -1,3 +1,21 @@
+2002-12-29  Helmer Kraemer  <hkraemer@freenet.de>
+
+	* kaffe/kaffevm/jni.c (getMethodFunc): new Method.
+	[JNI_METHOD_CODE]: removed.
+	(Kaffe_CallObjectMethod*), (Kaffe_CallBooleanMethod*),
+	(Kaffe_CallByteMethod*), (Kaffe_CallCharMethod*),
+	(Kaffe_CallShortMethod*), (Kaffe_CallIntMethod*),
+	(Kaffe_CallLongMethod*), (Kaffe_CallFloatMethod*),
+	(Kaffe_CallDoubleMethod*), (Kaffe_CallVoidMethod*):
+	properly handle interface methods.
+	Replaced JNI_METHOD_CODE with METHOD_INDIRECTMETHOD.
+	
+	* kaffe/kaffevm/support.c (callMethodV) [TRANSLATOR]:
+	removed bogus assertion.
+
+	* test/regression/ReflectInvoke.java: added tests for
+	invocation of interface and superclass methods.
+
 2002-12-10 Dalibor Topic <robilad@yahoo.com>
 
 	* configure.in: test for C++ compiler early on. Don't require
Index: kaffe/kaffevm/jni.c
===================================================================
RCS file: /cvs/kaffe/kaffe/kaffe/kaffevm/jni.c,v
retrieving revision 1.80
diff -w -u -p -r1.80 jni.c
--- kaffe/kaffevm/jni.c	18 Oct 2002 13:54:14 -0000	1.80
+++ kaffe/kaffevm/jni.c	29 Dec 2002 17:24:35 -0000
@@ -76,9 +76,27 @@ static void removeJNIref(jref);
 #endif
 
 /*
- * Define how we get the method to call.
+ * Find the function to be called when meth is invoked on obj
  */
-#define	JNI_METHOD_CODE(M)	METHOD_INDIRECTMETHOD(M)
+static inline void*
+getMethodFunc (Method* meth, Hjava_lang_Object *obj)
+{
+	if (obj && CLASS_IS_INTERFACE (meth->class)) {
+		register short *implementors;
+		register Hjava_lang_Class *clazz;
+		
+		assert (meth->idx >= 0);
+
+		implementors = meth->class->implementors;
+		clazz = OBJECT_CLASS(obj);
+
+		assert (implementors != NULL && clazz->impl_index <= implementors[0]);
+		
+		return clazz->itable2dtable[implementors[clazz->impl_index] + meth->idx + 1]; 	
+	} else {
+		return meth->idx >= 0 ? obj->dtable->method[meth->idx] : METHOD_INDIRECTMETHOD (meth);
+	}
+}
 
 /*
  * Define how to set the frame pointer in a VmExceptHandler.
@@ -475,7 +493,7 @@ Kaffe_NewObjectV(JNIEnv* env, jclass cls
 	}
 	obj = newObject(clazz);
 
-	callMethodV(m, JNI_METHOD_CODE(m), obj, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval);
 
 	ADD_REF(obj);
 	END_EXCEPTION_HANDLING();
@@ -515,7 +533,7 @@ Kaffe_NewObjectA(JNIEnv* env, jclass cls
 	}
 	obj = newObject(clazz);
 
-	callMethodA(m, JNI_METHOD_CODE(m), obj, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval, 0);
 
 	ADD_REF(obj);
 	END_EXCEPTION_HANDLING();
@@ -587,8 +605,7 @@ Kaffe_CallObjectMethodV(JNIEnv* env, job
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval);
+	callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
 
 	ADD_REF(retval.l);
 	END_EXCEPTION_HANDLING();
@@ -624,8 +641,7 @@ Kaffe_CallObjectMethodA(JNIEnv* env, job
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval, 0);
+	callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.l);
@@ -644,8 +660,7 @@ Kaffe_CallBooleanMethodV(JNIEnv* env, jo
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval);
+	callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return ((jboolean) retval.i);
@@ -680,8 +695,7 @@ Kaffe_CallBooleanMethodA(JNIEnv* env, jo
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval, 0);
+	callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return ((jboolean) retval.i);
@@ -700,8 +714,7 @@ Kaffe_CallByteMethodV(JNIEnv* env, jobje
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval);
+	callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return ((jbyte) retval.i);
@@ -736,8 +749,7 @@ Kaffe_CallByteMethodA(JNIEnv* env, jobje
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval, 0);
+	callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return ((jbyte) retval.i);
@@ -756,8 +768,7 @@ Kaffe_CallCharMethodV(JNIEnv* env, jobje
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval);
+	callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return ((jchar) retval.i);
@@ -792,8 +803,7 @@ Kaffe_CallCharMethodA(JNIEnv* env, jobje
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval, 0);
+	callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return ((jchar) retval.i);
@@ -812,8 +822,7 @@ Kaffe_CallShortMethodV(JNIEnv* env, jobj
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval);
+	callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return ((jshort) retval.i);
@@ -848,8 +857,7 @@ Kaffe_CallShortMethodA(JNIEnv* env, jobj
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval, 0);
+	callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return ((jshort) retval.i);
@@ -868,8 +876,7 @@ Kaffe_CallIntMethodV(JNIEnv* env, jobjec
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval);
+	callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.i);
@@ -904,8 +911,7 @@ Kaffe_CallIntMethodA(JNIEnv* env, jobjec
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval, 0);
+	callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.i);
@@ -924,8 +930,7 @@ Kaffe_CallLongMethodV(JNIEnv* env, jobje
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval);
+	callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.j);
@@ -960,8 +965,7 @@ Kaffe_CallLongMethodA(JNIEnv* env, jobje
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval, 0);
+	callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.j);
@@ -980,8 +984,7 @@ Kaffe_CallFloatMethodV(JNIEnv* env, jobj
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval);
+	callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.f);
@@ -1016,8 +1019,7 @@ Kaffe_CallFloatMethodA(JNIEnv* env, jobj
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval, 0);
+	callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.f);
@@ -1036,8 +1038,7 @@ Kaffe_CallDoubleMethodV(JNIEnv* env, job
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval);
+	callMethodV(m, getMethodFunc (m, o), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.d);
@@ -1072,8 +1073,7 @@ Kaffe_CallDoubleMethodA(JNIEnv* env, job
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, &retval, 0);
+	callMethodA(m, getMethodFunc (m, o), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.d);
@@ -1091,8 +1091,7 @@ Kaffe_CallVoidMethodV(JNIEnv* env, jobje
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, 0);
+	callMethodV(m, getMethodFunc (m, o), obj, args, 0);
 
 	END_EXCEPTION_HANDLING();
 }
@@ -1123,8 +1122,7 @@ Kaffe_CallVoidMethodA(JNIEnv* env, jobje
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, (m->idx >= 0 ? o->dtable->method[m->idx]
-			: JNI_METHOD_CODE(m)), obj, args, 0, 0);
+	callMethodA(m, getMethodFunc (m, o), obj, args, 0, 0);
 
 	END_EXCEPTION_HANDLING();
 }
@@ -1141,7 +1139,7 @@ Kaffe_CallNonvirtualObjectMethodV(JNIEnv
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), obj, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval);
 
 	ADD_REF(retval.l);
 	END_EXCEPTION_HANDLING();
@@ -1176,7 +1174,7 @@ Kaffe_CallNonvirtualObjectMethodA(JNIEnv
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), obj, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval, 0);
 
 	ADD_REF(retval.l);
 	END_EXCEPTION_HANDLING();
@@ -1195,7 +1193,7 @@ Kaffe_CallNonvirtualBooleanMethodV(JNIEn
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), obj, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return ((jboolean) retval.i);
@@ -1229,7 +1227,7 @@ Kaffe_CallNonvirtualBooleanMethodA(JNIEn
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), obj, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return ((jboolean) retval.i);
@@ -1247,7 +1245,7 @@ Kaffe_CallNonvirtualByteMethodV(JNIEnv* 
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), obj, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return ((jbyte) retval.i);
@@ -1281,7 +1279,7 @@ Kaffe_CallNonvirtualByteMethodA(JNIEnv* 
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), obj, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return ((jbyte) retval.i);
@@ -1299,7 +1297,7 @@ Kaffe_CallNonvirtualCharMethodV(JNIEnv* 
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), obj, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return ((jchar) retval.i);
@@ -1333,7 +1331,7 @@ Kaffe_CallNonvirtualCharMethodA(JNIEnv* 
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), obj, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return ((jchar) retval.i);
@@ -1351,7 +1349,7 @@ Kaffe_CallNonvirtualShortMethodV(JNIEnv*
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), obj, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return ((jshort) retval.i);
@@ -1385,7 +1383,7 @@ Kaffe_CallNonvirtualShortMethodA(JNIEnv*
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), obj, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return ((jshort) retval.i);
@@ -1403,7 +1401,7 @@ Kaffe_CallNonvirtualIntMethodV(JNIEnv* e
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), obj, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.i);
@@ -1437,7 +1435,7 @@ Kaffe_CallNonvirtualIntMethodA(JNIEnv* e
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), obj, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.i);
@@ -1455,7 +1453,7 @@ Kaffe_CallNonvirtualLongMethodV(JNIEnv* 
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), obj, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.j);
@@ -1489,7 +1487,7 @@ Kaffe_CallNonvirtualLongMethodA(JNIEnv* 
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), obj, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.j);
@@ -1507,7 +1505,7 @@ Kaffe_CallNonvirtualFloatMethodV(JNIEnv*
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), obj, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.f);
@@ -1541,7 +1539,7 @@ Kaffe_CallNonvirtualFloatMethodA(JNIEnv*
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), obj, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.f);
@@ -1559,7 +1557,7 @@ Kaffe_CallNonvirtualDoubleMethodV(JNIEnv
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), obj, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.d);
@@ -1593,7 +1591,7 @@ Kaffe_CallNonvirtualDoubleMethodA(JNIEnv
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), obj, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), obj, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.d);
@@ -1610,7 +1608,7 @@ Kaffe_CallNonvirtualVoidMethodV(JNIEnv* 
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), obj, args, 0);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), obj, args, 0);
 
 	END_EXCEPTION_HANDLING();
 }
@@ -1640,7 +1638,7 @@ Kaffe_CallNonvirtualVoidMethodA(JNIEnv* 
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), obj, args, 0, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), obj, args, 0, 0);
 
 	END_EXCEPTION_HANDLING();
 }
@@ -1896,7 +1894,7 @@ Kaffe_CallStaticObjectMethodV(JNIEnv* en
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), 0, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval);
 
 	ADD_REF(retval.l);
 	END_EXCEPTION_HANDLING();
@@ -1931,7 +1929,7 @@ Kaffe_CallStaticObjectMethodA(JNIEnv* en
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), 0, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval, 0);
 
 	ADD_REF(retval.l);
 	END_EXCEPTION_HANDLING();
@@ -1950,7 +1948,7 @@ Kaffe_CallStaticBooleanMethodV(JNIEnv* e
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), 0, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return ((jboolean) retval.i);
@@ -1984,7 +1982,7 @@ Kaffe_CallStaticBooleanMethodA(JNIEnv* e
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), 0, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return ((jboolean) retval.i);
@@ -2002,7 +2000,7 @@ Kaffe_CallStaticByteMethodV(JNIEnv* env,
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), 0, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return ((jbyte) retval.i);
@@ -2036,7 +2034,7 @@ Kaffe_CallStaticByteMethodA(JNIEnv* env,
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), 0, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return ((jbyte) retval.i);
@@ -2054,7 +2052,7 @@ Kaffe_CallStaticCharMethodV(JNIEnv* env,
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), 0, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return ((jchar) retval.i);
@@ -2088,7 +2086,7 @@ Kaffe_CallStaticCharMethodA(JNIEnv* env,
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), 0, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return ((jchar) retval.i);
@@ -2106,7 +2104,7 @@ Kaffe_CallStaticShortMethodV(JNIEnv* env
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), 0, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return ((jshort) retval.i);
@@ -2140,7 +2138,7 @@ Kaffe_CallStaticShortMethodA(JNIEnv* env
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), 0, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return ((jshort) retval.i);
@@ -2158,7 +2156,7 @@ Kaffe_CallStaticIntMethodV(JNIEnv* env, 
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), 0, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.i);
@@ -2192,7 +2190,7 @@ Kaffe_CallStaticIntMethodA(JNIEnv* env, 
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), 0, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.i);
@@ -2210,7 +2208,7 @@ Kaffe_CallStaticLongMethodV(JNIEnv* env,
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), 0, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.j);
@@ -2244,7 +2242,7 @@ Kaffe_CallStaticLongMethodA(JNIEnv* env,
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), 0, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.j);
@@ -2262,7 +2260,7 @@ Kaffe_CallStaticFloatMethodV(JNIEnv* env
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), 0, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.f);
@@ -2296,7 +2294,7 @@ Kaffe_CallStaticFloatMethodA(JNIEnv* env
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), 0, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.f);
@@ -2314,7 +2312,7 @@ Kaffe_CallStaticDoubleMethodV(JNIEnv* en
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), 0, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.d);
@@ -2348,7 +2346,7 @@ Kaffe_CallStaticDoubleMethodA(JNIEnv* en
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), 0, args, &retval, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval, 0);
 
 	END_EXCEPTION_HANDLING();
 	return (retval.d);
@@ -2366,7 +2364,7 @@ Kaffe_CallStaticVoidMethodV(JNIEnv* env,
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodV(m, JNI_METHOD_CODE(m), 0, args, &retval);
+	callMethodV(m, METHOD_INDIRECTMETHOD(m), 0, args, &retval);
 
 	END_EXCEPTION_HANDLING();
 }
@@ -2396,7 +2394,7 @@ Kaffe_CallStaticVoidMethodA(JNIEnv* env,
 		throwException(NoSuchMethodError(m->name->data));
 	}
 
-	callMethodA(m, JNI_METHOD_CODE(m), 0, args, 0, 0);
+	callMethodA(m, METHOD_INDIRECTMETHOD(m), 0, args, 0, 0);
 
 	END_EXCEPTION_HANDLING();
 }
Index: test/regression/ReflectInvoke.java
===================================================================
RCS file: /cvs/kaffe/kaffe/test/regression/ReflectInvoke.java,v
retrieving revision 1.2
diff -w -u -p -r1.2 ReflectInvoke.java
--- test/regression/ReflectInvoke.java	12 Feb 1999 13:51:10 -0000	1.2
+++ test/regression/ReflectInvoke.java	29 Dec 2002 17:24:36 -0000
@@ -6,6 +6,22 @@ import java.lang.reflect.*;
  */
 public class ReflectInvoke {
 
+  static interface testInterface {
+    void a ();
+  }
+
+  static class testImplBase implements testInterface {
+    public void a () {
+      System.out.println ("Success_Super");
+    }
+  }
+
+  static class testImplSub extends testImplBase {
+    public void a () {
+      System.out.println ("Success_Sub");
+    }
+  }
+ 
   public int dd(int i)	{ return 2 * i; }
   public static int DD(int i)	{ return i/2; }
 
@@ -23,6 +39,16 @@ public class ReflectInvoke {
 
       o = c.getMethod("dd", argtypes).invoke(c.newInstance(), args);
       System.out.println(((Integer)o).intValue());
+
+      Method m = testInterface.class.getMethod ("a", null);
+
+      m.invoke (new testImplBase(), null);
+      
+      m.invoke (new testImplSub(), null);
+
+      m = testImplBase.class.getMethod ("a", null);
+      
+      m.invoke (new testImplSub(), null);
     } catch (Exception e) {
       System.out.println("caught " + e);
       e.printStackTrace(System.out);
@@ -33,4 +59,7 @@ public class ReflectInvoke {
 /* Expected Output:
 2
 8
+Success_Super
+Success_Sub
+Success_Sub
 */
